#!/bin/sh
# srcdir may get passed in by autoconf
#
if [ "$srcdir" != "" ]; then
	srcdir=$srcdir"/"
else
	srcdir="/"
fi

srcdir="`pwd`/$srcdir"

# Test the locally built versions, in preference to the installed ones.
# Also, there might not be anything installed as yet (eg. during "make check")
export PATH=../src:../scripts:$PATH

# Source some functions
. $srcdir"testfunctions"

echo -n "Test security issues with default dependencies  ... "
directory=$srcdir"2"

tmpfile=`mktemp -q /tmp/XXXXXX`
tmpconfigfile=`mktemp -q /tmp/XXXXXX`

if [ ! -w $tmpfile -o ! -w $tmpconfigfile ]; then
	echo Unable to create temporary file.
	failure
fi

# build the serel config file on the fly.
# We only go though this gynastic because relative pathnames
# are meaningless under autoconf's "make distcheck"
#
cat <<EOF > $tmpconfigfile
# Test #2
#
rdfpath = $srcdir/../data/serel-default.rdf
dbignore = yes
EOF

startServer -c $tmpconfigfile
[ $? != "0" ] && failure
# exit

# always stop the server, no matter what
trap "serelc --control stop > /dev/null 2>&1" EXIT

for i in ipchains iptables network; do
	if [ ! -e "$directory/$i" ]; then
		# echo ln -s "$directory"/anacron "$directory"/$i
		ln -s "$directory"/anacron "$directory"/$i
	fi
done

SEREL_PAIRS=
for i in "$directory"/*; do
	if [ ! -d $i ]; then
		service=`basename $i`
		SEREL_PAIRS="$SEREL_PAIRS $service $i" 
	fi
done

serelhistory --populate << END
--pairs
$SEREL_PAIRS
END
[ $? != "0" ] && failure

serelhistory --query verify > /dev/null
[ $? != "0" ] && failure

for i in "$directory"/*; do
	[ ! -d $i ] && serel-wrap $i start &
done

wait

testStatus $directory "eStateIn"

serelc --query graph > $tmpfile
[ $? != "0" ] && failure

# Test that network depends on iptables
#
grep -q edge-iptables-network $tmpfile
[ $? != "0" ] && failure

# Test that network depends on ipchains
#
grep -q edge-ipchains-network $tmpfile
[ $? != "0" ] && failure

rm $tmpfile $tmpconfigfile

serelc --control stop

success
